/*****************************************************************************
 * Copyright (c) 2014-2026 OpenRCT2 developers
 *
 * For a complete list of all authors, please refer to contributors.md
 * Interested in contributing? Visit https://github.com/OpenRCT2/OpenRCT2
 *
 * OpenRCT2 is licensed under the GNU General Public License version 3.
 *****************************************************************************/

#pragma once

#include <cstdint>

namespace OpenRCT2
{
    enum class TileElementType : uint8_t
    {
        surface = 0,
        path = 1,
        track = 2,
        smallScenery = 3,
        entrance = 4,
        wall = 5,
        largeScenery = 6,
        banner = 7,
    };
}
